import re
from . import API_BASE_URL
from .scores import calculate_score

# pylint: disable=old-style-class
class Thread:
    def __init__(self, title, dealer_name, score, url, views):
        self.dealer_name = dealer_name
        self.score = score
        self.title = title
        self.url = url
        self.views = views

    def __repr__(self):
        return "Thread(%s)" % self.title


def build_web_path(slug):
    return "{}{}".format(API_BASE_URL, slug)


def get_dealer(topic):
    dealer = None
    if topic.get("offer"):
        dealer = topic.get("offer").get("dealer_name")
    return dealer


def parse_threads(threads, limit):
    """Parse topics list api response into digestible list.

    Arguments:
        threads {dict} -- topics response from rfd api
        limit {int} -- limit number of threads returned

    Returns:
        list(dict) -- digestible list of threads
    """
    parsed_threads = []
    if threads is None:
        return []
    for count, topic in enumerate(threads.get("topics"), start=1):
        if count > limit:
            break
        parsed_threads.append(
            Thread(
                title=topic.get("title"),
                dealer_name=get_dealer(topic),
                score=calculate_score(topic),
                url=build_web_path(topic.get("web_path")),
                views=topic.get("total_views"),
            )
        )
    return parsed_threads


def sort_threads(threads, sort_by):
    """Sort threads by an attribute"""
    if sort_by is None:
        return threads
    assert sort_by in ["views", "score", "title"]
    threads = sorted(threads, key=lambda x: getattr(x, sort_by), reverse=True)
    return threads


def search_threads(threads, regex):
    """Match deal title and dealer names with regex specified."""

    regexp = re.compile(str(regex).lower())

    for deal in threads:

        if regexp.search(deal.title.lower()) or (
            deal.dealer_name and regexp.search(deal.dealer_name.lower())
        ):
            yield deal



from __future__ import unicode_literals


import logging
import os
import sys
import click
from colorama import init, Fore, Style
from .api import get_threads, get_posts
from .threads import parse_threads, search_threads, sort_threads
from .__version__ import version as current_version

init()
print()

logging.getLogger()
logging.getLogger().setLevel(logging.INFO)
logging.getLogger().addHandler(logging.StreamHandler())


def get_version():
    return "rfd v" + current_version


def get_terminal_width():
    _, columns = os.popen("stty size", "r").read().split()
    return int(columns)


def get_vote_color(score):
    if score > 0:
        return Fore.GREEN + " [+" + str(score) + "] "
    if score < 0:
        return Fore.RED + " [" + str(score) + "] "
    return Fore.BLUE + " [" + str(score) + "] "


def print_version(ctx, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(get_version(), nl=False)
    ctx.exit()


def generate_thread_output(_threads):
    for count, thread in enumerate(_threads, 1):
        output = ""
        dealer = thread.dealer_name
        if dealer and dealer is not None:
            dealer = "[" + dealer + "] "
        else:
            dealer = ""
        output += (
            " "
            + str(count)
            + "."
            + get_vote_color(thread.score)
            + Fore.RESET
            + "%s%s" % (dealer, thread.title)
            + Fore.LIGHTYELLOW_EX
            + " (%d views)" % thread.views
            + Fore.RESET
        )
        output += Fore.BLUE + " {}".format(thread.url)
        output += Style.RESET_ALL
        output += "\n\n"
        yield output


@click.group(invoke_without_command=True)
@click.option(
    "-v",
    "--version",
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
)
@click.pass_context
def cli(ctx):
    """CLI for https://forums.redflagdeals.com"""
    if not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


@cli.command(short_help="Display all posts in a thread.")
@click.argument("post_id")
def posts(post_id):
    """Iterate all pages and display all posts in a thread.

    post_id can be a full url or post id only

    Example:

    \b
    rfd posts https://forums.redflagdeals.com/koodo-targeted-public-mobile-12-120-koodo-5gb-40-no-referrals-2173603
    """

    try:
        click.echo("-" * get_terminal_width())
        for post in get_posts(post=post_id):
            click.echo(
                " -"
                + get_vote_color(post.score)
                + Fore.RESET
                + post.body
                + Fore.YELLOW
                + " ({})".format(post.user)
            )
            click.echo(Style.RESET_ALL)
            click.echo("-" * get_terminal_width())
    except ValueError:
        click.echo("Invalid post id.")
        sys.exit(1)
    except AttributeError:
        click.echo("The RFD API did not return the expected data.")
        sys.exit(1)


@cli.command(short_help="Displays threads in the forum. Defaults to hot deals.")
@click.option("--forum-id", default=9, help="The forum id number")
@click.option("--limit", default=10, help="Number of threads.")
@click.option("--sort-by", default=None, help="Sort threads by")
def threads(limit, forum_id, sort_by):
    """Display threads in the specified forum id. Defaults to 9 (hot deals).

    Popular forum ids:

    \b
    9 \t hot deals
    14 \t computer and electronics
    15 \t offtopic
    17 \t entertainment
    18 \t food and drink
    40 \t automotive
    53 \t home and garden
    67 \t fashion and apparel
    74 \t shopping discussion
    88 \t cell phones
    """
    _threads = sort_threads(
        parse_threads(get_threads(forum_id, limit), limit), sort_by=sort_by
    )
    click.echo_via_pager(generate_thread_output(_threads))


@cli.command(short_help="Search deals based on a regular expression.")
@click.option("--pages", default=5, help="Number of pages to search.")
@click.option(
    "--forum-id", default=9, help="The forum id number. Defaults to 9 (hot deals)."
)
@click.option("--sort-by", default=None, help="Sort threads by")
@click.argument("regex")
def search(pages, forum_id, sort_by, regex):
    """Search deals based on regex.

    Popular forum ids:

    \b
    9 \t hot deals
    14 \t computer and electronics
    15 \t offtopic
    17 \t entertainment
    18 \t food and drink
    40 \t automotive
    53 \t home and garden
    67 \t fashion and apparel
    74 \t shopping discussion
    88 \t cell phones
    """

    matched_threads = []

    for page in range(1, pages):
        _threads = parse_threads(get_threads(forum_id, 100, page=page), limit=100)
        for thread in search_threads(threads=_threads, regex=regex):
            matched_threads.append(thread)
    click.echo_via_pager(
        generate_thread_output(sort_threads(matched_threads, sort_by=sort_by))
    )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__title__ = "RFD CLI"
__author__ = "Dave Gallant"
__license__ = "Apache 2.0"
__copyright__ = "(c) 2018 Dave Gallant"

API_BASE_URL = "https://forums.redflagdeals.com"



"""RFD API."""

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError
import logging
import requests
from . import API_BASE_URL
from .posts import Post
from .scores import calculate_score
from .utils import is_int, strip_html, is_valid_url


def extract_post_id(url):
    return url.split("/")[3].split("-")[-1]


def get_safe_per_page(limit):
    """Ensure that per page limit is between 5-40"""
    if limit < 5:
        return 5
    if limit > 40:
        return 40
    return limit


def create_user_map(users):
    """Create a map of user ids to usernames."""
    m = dict()
    for user in users:
        m[user.get("user_id")] = user.get("username")
    return m


def get_threads(forum_id, limit, page=1):
    """Get threads from rfd api

    Arguments:
        forum_id {int} -- forum id
        limit {[type]} -- limit number of threads returned

    Returns:
        dict -- api response
    """
    try:
        response = requests.get(
            "{}/api/topics?forum_id={}&per_page={}&page={}".format(
                API_BASE_URL, forum_id, get_safe_per_page(limit), page
            )
        )
        if response.status_code == 200:
            return response.json()
        logging.error("Unable to retrieve threads. %s", response.text)
    except JSONDecodeError as err:
        logging.error("Unable to retrieve threads. %s", err)
    return None


def get_posts(post):
    """Retrieve posts from a thread.

    Args:
        post (str): either full url or postid

    Yields:
        list(Post): Posts
    """
    if is_valid_url(post):
        post_id = extract_post_id(post)
    elif is_int(post):
        post_id = post
    else:
        raise ValueError()

    response = requests.get(
        "{}/api/topics/{}/posts?per_page=40&page=1".format(API_BASE_URL, post_id)
    )

    total_pages = response.json().get("pager").get("total_pages")

    for page in range(0, total_pages + 1):
        response = requests.get(
            "{}/api/topics/{}/posts?per_page={}&page={}".format(
                API_BASE_URL, post_id, 40, page
            )
        )
        users = create_user_map(response.json().get("users"))

        posts = response.json().get("posts")

        for i in posts:
            # Sometimes votes is null
            if i.get("votes") is not None:
                calculated_score = calculate_score(i)
            else:
                calculated_score = 0
            yield Post(
                body=strip_html(i.get("body")),
                score=calculated_score,
                user=users[i.get("author_id")],
            )



from rfd.api import extract_post_id
from rfd.threads import build_web_path, parse_threads


def test_build_web_path():
    assert build_web_path("/test") == "https://forums.redflagdeals.com/test"


def test_extract_post_id():
    assert (
        extract_post_id("https://forums.redflagdeals.com/targeted-bob-2173603/120")
        == "2173603"
    )
    assert (
        extract_post_id("http://forums.redflagdeals.com/targeted-2173604/120")
        == "2173604"
    )


def test_parse_threads(threads_api_response):

    limit = 10
    threads = parse_threads(threads_api_response, limit)
    assert len(threads) == limit


def test_parse_threads_empty():

    assert parse_threads(None, 10) == []



def calculate_score(post):
    """Calculate either topic or post score. If votes cannot be retrieved, the score is 0.

    Arguments:
        post {dict} -- pass in the topic/post object

    Returns:
        int -- score
    """
    score = 0
    try:
        score = int(post.get("votes").get("total_up")) - int(
            post.get("votes").get("total_down")
        )
    except AttributeError:
        pass

    return score



import io  # for python2
from os import path
from setuptools import setup, find_packages
from rfd.__version__ import version


def load_requirements(file_name):
    """ Reads and returns requirements """
    with io.open(path.join(WORKING_DIR, file_name)) as handle:
        raw_reqs = handle.read().strip().split("\n")
        reqs = [req for req in raw_reqs]
        return reqs


WORKING_DIR = path.abspath(path.dirname(__file__))
REQUIREMENTS = load_requirements("requirements.txt")
TEST_REQUIREMENTS = load_requirements("requirements_dev.txt")

# Get long description from README.md
with io.open(path.join(WORKING_DIR, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    author="Dave Gallant",
    description="CLI for RedFlagDeals.com",
    entry_points={"console_scripts": ["rfd = rfd.__main__:cli"]},
    include_package_data=True,
    install_requires=REQUIREMENTS,
    keywords="cli redflagdeals",
    license="Apache License, Version 2.0",
    long_description=long_description,
    long_description_content_type="text/markdown",
    name="rfd",
    packages=find_packages(),
    url="https://github.com/davegallant/rfd",
    version=version,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from os.path import dirname, abspath, join


def load_version():
    with open(join(dirname(abspath(__file__)), "VERSION")) as handle:
        return handle.read()


version = load_version()
